"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.mts
var index_exports = {};
__export(index_exports, {
  default: () => index_default,
  every: () => every,
  filter: () => filter,
  forEach: () => forEach,
  some: () => some,
  traverse: () => traverse2
});
module.exports = __toCommonJS(index_exports);
var import_path = __toESM(require("path"), 1);
var import_fs = __toESM(require("fs"), 1);
function makeAsync(func) {
  return async (arg) => func(arg);
}
function makeFilter(test) {
  if (!test) {
    return false;
  }
  if (test instanceof RegExp) {
    return test;
  }
  try {
    return new RegExp(test);
  } catch (e) {
    throw new Error(`Invalid regular expression: ${test}`);
  }
}
function makeInclude(include, exclude) {
  const includeFilter = makeFilter(include);
  const excludeFilter = makeFilter(exclude);
  return function(filepath) {
    if (includeFilter && !includeFilter.test(filepath)) {
      return false;
    }
    if (excludeFilter && excludeFilter.test(filepath)) {
      return false;
    }
    return true;
  };
}
async function search(match, context, directory, state = { complete: false }) {
  const { fs, basedir, includeFile, includeDirectory } = context;
  if (state.complete) {
    return;
  }
  const entries = await fs.promises.readdir(import_path.default.join(basedir, directory), {
    encoding: "utf8",
    withFileTypes: true
  });
  const files = entries.filter((entry) => entry.isFile() && includeFile(entry.name)).map((entry) => import_path.default.join(directory, entry.name));
  const directories = entries.filter((entry) => entry.isDirectory() && includeDirectory(entry.name)).map((entry) => import_path.default.join(directory, entry.name));
  const fileSearch = files.map(async (file) => {
    if (!state.complete) {
      const result = await match(file);
      if (result) {
        state.complete = true;
      }
    }
  });
  const directorySearch = directories.map(
    (directory2) => search(match, context, directory2, state)
  );
  await Promise.all([...fileSearch, ...directorySearch]);
}
function optionsToContext(options) {
  const basedir = import_path.default.isAbsolute(options.directory || "") ? options.directory || "" : import_path.default.resolve(import_path.default.join(process.cwd(), options.directory || ""));
  const includeFile = makeInclude(options.include_file, options.exclude_file);
  const includeDirectory = makeInclude(
    options.include_dir,
    options.exclude_dir
  );
  const fs = options.fs || import_fs.default;
  return {
    fs,
    basedir,
    includeFile,
    includeDirectory
  };
}
var _traverse = async function traverse(options) {
  if (typeof options.process_file !== "function") {
    throw new Error("process_file must be a function");
  }
  const processFile = options.process_file;
  const context = optionsToContext(options);
  const results = [];
  await search(
    async (file) => {
      results.push(processFile(file));
      return false;
    },
    context,
    "./"
  );
  return results;
};
async function some(match, options) {
  if (typeof match != "function") {
    throw new Error("match must be a function");
  }
  const asyncMatch = makeAsync(match);
  const context = optionsToContext(options);
  const state = { complete: false };
  await search(asyncMatch, context, "./", state);
  return state.complete;
}
async function every(match, options) {
  if (typeof match !== "function") {
    throw new Error("match must be a function");
  }
  const asyncMatch = makeAsync(match);
  const context = optionsToContext(options);
  let every2 = true;
  await search(
    async (file) => {
      if (!every2) {
        return true;
      }
      const isMatch = await asyncMatch(file);
      if (!isMatch) {
        every2 = false;
        return true;
      }
      return false;
    },
    context,
    "./"
  );
  return every2;
}
async function filter(match, options) {
  if (typeof match !== "function") {
    throw new Error("match must be a function");
  }
  const asyncMatch = makeAsync(match);
  const context = optionsToContext(options);
  const results = [];
  await search(
    async (file) => {
      const isMatch = await asyncMatch(file);
      if (isMatch) {
        results.push(file);
      }
      return false;
    },
    context,
    "./"
  );
  return results;
}
async function forEach(apply, options) {
  if (typeof apply !== "function") {
    throw new Error("match must be a function");
  }
  const asyncApply = makeAsync(apply);
  const context = optionsToContext(options);
  await search(
    async (file) => {
      await asyncApply(file);
      return false;
    },
    context,
    "./"
  );
}
var traverse2 = Object.assign(_traverse, {
  traverse: _traverse,
  some,
  every,
  filter,
  forEach
});
var index_default = traverse2;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  every,
  filter,
  forEach,
  some,
  traverse
});
//# sourceMappingURL=index.js.map
